@@ -1,10 +1,11 @@
-/* @(#)raw.c	1.2 99/12/19 Copyright 1998,1999 Heiko Eissfeldt */
+/* @(#)raw.c	1.3 00/03/26 Copyright 1998,1999 Heiko Eissfeldt */
 #ifndef lint
 static char     sccsid[] =
-"@(#)raw.c	1.2 99/12/19 Copyright 1998,1999 Heiko Eissfeldt";
+"@(#)raw.c	1.3 00/03/26 Copyright 1998,1999 Heiko Eissfeldt";
 
 #endif
 #include "config.h"
+#include <unistd.h>
 #include "sndfile.h"
 
 static int InitSound __PR(( void ));
@@ -28,6 +29,13 @@ static unsigned long GetHdrSize( )
   return 0L;
 }
 
+static unsigned long InSizeToOutSize __PR(( unsigned long BytesToDo ));
+
+static unsigned long InSizeToOutSize ( BytesToDo )
+        unsigned long BytesToDo;
+{
+        return BytesToDo;
+}
 
 struct soundfile rawsound =
 {
@@ -39,5 +47,9 @@ struct soundfile rawsound =
 
   GetHdrSize,
 
+  (int (*) __PR(( int audio, unsigned char *buf, unsigned long BytesToDo ))) write,		/* get sound samples out */
+
+  InSizeToOutSize,	/* compressed? output file size */
+
   1		/* needs big endian samples */
 };
