@@ -1,3 +1,8 @@
+/* @(#)isovfy.c	1.13 99/12/27 joerg */
+#ifndef lint
+static	char sccsid[] =
+	"@(#)isovfy.c	1.13 99/12/27 joerg";
+#endif
 /*
  * File isovfy.c - verify consistency of iso9660 filesystem.
  *
@@ -20,8 +25,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-static char rcsid[] ="$Id: isovfy.c,v 1.5 1999/03/02 03:41:36 eric Exp $";
-
 #include "../config.h"
 
 #include <stdxlib.h>
